Clubhouse = {};

local f = CreateFrame('frame');
local currentRegionName = GetCurrentRegionName();
local currentFactionName = UnitFactionGroup("player");
local urlMatch;

if currentRegionName == "CN" then
  urlMatch = COMMUNITIES_INVITE_MANAGER_TICKET_FORMAT_CHARACTER_CN:gsub("%%s","(%%w+)")
else
  urlMatch = COMMUNITIES_INVITE_MANAGER_TICKET_FORMAT_CHARACTER:gsub("%%s","(%%w+)")
end

function Clubhouse:AddInvite(ticketId, itemLink, itemLinkText)
  local inviteInfo = { itemLink = itemLink, region = currentRegionName, faction = currentFactionName, time = time() };
  if not Clubhouse_Invites[currentRegionName][currentFactionName][ticketId] then
    Clubhouse_Invites[currentRegionName][currentFactionName][ticketId] = inviteInfo;
    DEFAULT_CHAT_FRAME:AddMessage(string.format("[Clubhouse] detected invitation: %s", itemLinkText));
  end
end

function Clubhouse:AddFilters()
  function ClubhouseFilter(self, event, message, author, ...)
    -- https://www.worldofwarcraft.com/invite/foo?region=US&faction=Horde
    local ticketId, regionName, faction = message:match(urlMatch);

    if ticketId then
      local itemLinkText = author .. " " .. ticketId;
      local itemLink = string.format("|cffffd100|HclubTicket:%s|h"..CLUB_INVITE_HYPERLINK_TEXT.."|h|r", ticketId, itemLinkText);
      Clubhouse:AddInvite(ticketId, itemLink, itemLinkText);
    end

    -- [Join: foo]
    for itemLink in message:gmatch("|c%x+|HclubTicket:[^|]+|h[^|]+|h|r") do
      local ticketId, clubInviteText = itemLink:match("|c%x+|HclubTicket:([^|]+)|h([^|]+)|h|r");
      Clubhouse:AddInvite(ticketId, itemLink, clubInviteText);
    end
  end

  ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_CONVERSATION",       ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER",            ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER_INFORM",     ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_BATTLEGROUND",          ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_BATTLEGROUND_LEADER",   ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL",               ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_COMMUNITIES_CHANNEL",   ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_EMOTE",                 ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD",                 ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_INSTANCE_CHAT",         ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_INSTANCE_CHAT_LEADER",  ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_OFFICER",               ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY",                 ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY_LEADER",          ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID",                  ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_LEADER",           ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_WARNING",          ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY",                   ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_TEXT_EMOTE",            ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER",               ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM",        ClubhouseFilter);
  ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL",                  ClubhouseFilter);
end

SLASH_CLUBHOUSE1 = '/clubhouse';
function SlashCmdList.CLUBHOUSE(msg)
  local _, _, cmd, args = string.find(msg, "%s?(%w+)%s?(.*)");
  if cmd == 'clear' then
    -- todo - clear user specified invites ?
    if args == 'all' then
      Clubhouse_Invites = {}
      DEFAULT_CHAT_FRAME:AddMessage("[Clubhouse]: cleared ALL known invites");
    else
      Clubhouse_Invites[currentRegionName][currentFactionName] = {}
      DEFAULT_CHAT_FRAME:AddMessage("[Clubhouse]: cleared known invites");
    end
  elseif cmd == 'days' then
    local days = tonumber(args)
    if not args then
      DEFAULT_CHAT_FRAME:AddMessage("[Clubhouse]: retention currently set to "..args.." day(s)");
    elseif days > 0 and days <= 365 then
      DEFAULT_CHAT_FRAME:AddMessage("[Clubhouse]: retention set to "..args.." day(s)");
    else
      DEFAULT_CHAT_FRAME:AddMessage("[Clubhouse]: invalid retention days");
    end
  elseif cmd == 'list' then
    if args == 'all' then
      for region in pairs(Clubhouse_Invites) do
        for faction in pairs(Clubhouse_Invites[region]) do
          for ticketId, inviteInfo in pairs(Clubhouse_Invites[currentRegionName][currentFactionName]) do
            DEFAULT_CHAT_FRAME:AddMessage("[Clubhouse] invite for region ["..inviteInfo.region.."] and faction ["..inviteInfo.faction.."]: " .. inviteInfo.itemLink);
          end
        end
      end
    else
      for ticketId, inviteInfo in pairs(Clubhouse_Invites[currentRegionName][currentFactionName]) do
        DEFAULT_CHAT_FRAME:AddMessage("[Clubhouse] invite for region ["..inviteInfo.region.."] and faction ["..inviteInfo.faction.."]: " .. inviteInfo.itemLink);
      end
    end
  else
    DEFAULT_CHAT_FRAME:AddMessage("[Clubhouse] current invite retention is "..Clubhouse_MaxDays.." day(s)");
    DEFAULT_CHAT_FRAME:AddMessage(SLASH_CLUBHOUSE1 .. " days <maximum invite retention in days 1-365>");
    DEFAULT_CHAT_FRAME:AddMessage(SLASH_CLUBHOUSE1 .. " clear -- removes known invites for this region+faction");
    DEFAULT_CHAT_FRAME:AddMessage(SLASH_CLUBHOUSE1 .. " clear all -- removes ALL known invites");
    DEFAULT_CHAT_FRAME:AddMessage(SLASH_CLUBHOUSE1 .. " list -- show invites for this region+faction");
    DEFAULT_CHAT_FRAME:AddMessage(SLASH_CLUBHOUSE1 .. " list all -- show all invites");
  end
end

f:SetScript('OnEvent', function(self, event, ...)
  if event == 'ADDON_LOADED' then
    Clubhouse:AddFilters();
    f:UnregisterEvent("ADDON_LOADED")
  elseif event == 'PLAYER_LOGIN' then
    if not Clubhouse_MaxDays then
      Clubhouse_MaxDays = 3
    end
    if not Clubhouse_Invites then
      Clubhouse_Invites = {}
    end
    if not Clubhouse_Invites[currentRegionName] then
      Clubhouse_Invites[currentRegionName] = {}
    end
    if not Clubhouse_Invites[currentRegionName][currentFactionName] then
      Clubhouse_Invites[currentRegionName][currentFactionName] = {}
    else
      for ticketId, inviteInfo in pairs(Clubhouse_Invites[currentRegionName][currentFactionName]) do
        if time() - inviteInfo.time > 86400*Clubhouse_MaxDays then
          Clubhouse_Invites[currentRegionName][currentFactionName][ticketId] = nil;
        elseif inviteInfo.region == currentRegionName and inviteInfo.faction == currentFactionName then
          DEFAULT_CHAT_FRAME:AddMessage("[Clubhouse] Community invitation: " .. inviteInfo.itemLink);
        end
      end
    end
  end
end)
f:RegisterEvent('ADDON_LOADED');
f:RegisterEvent('PLAYER_LOGIN');